package com.netease.nim.uikit.business.redpacket

import android.support.v4.app.FragmentActivity
import android.text.SpannableString
import android.text.format.DateUtils
import android.text.style.ImageSpan
import android.view.LayoutInflater
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.preference.RedPacketConfig
import com.netease.nim.uikit.business.uinfo.UserInfoHelper
import com.netease.nim.uikit.common.presenter.AbsPresenter
import com.netease.nim.uikit.common.rx.LoadingDisposableCollector
import com.netease.nim.uikit.impl.database.FlashAppDBHelper
import com.netease.nim.uikit.impl.database.RedPacketCheckerEntity
import com.smartisan.libstyle.dialog.BulletAlertCustomViewDialog
import com.tencent.mm.opensdk.utils.Log
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.functions.Consumer
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.amount_limit_check.view.*


class AmountChecker : AbsPresenter() {

    companion object {
        private var TAG = AmountChecker::class.java.simpleName
        private const val TIME = DateUtils.DAY_IN_MILLIS
        private const val MAX_AMOUNT = 200.0
        private const val ADDITIVE_MAX_AMOUNT = 5000.0
    }

    private var loadingDisposableCollector: LoadingDisposableCollector? = null
    private var currentChecker: RedPacketCheckerEntity? = null
    private var checkerType: CheckerType = CheckerType.None

    override fun onAttach(activity: FragmentActivity?) {
        super.onAttach(activity)
        loadingDisposableCollector = LoadingDisposableCollector(activity)
    }

    override fun onDetach() {
        super.onDetach()
        loadingDisposableCollector?.clearDisposable()
    }

    fun checkAmount(account: String?, amount: String?, callback: CheckResultCallback) {
        amount ?: return
        account ?: return
        checkAmount(account, amount.toDoubleOrNull(), callback)
    }

    private fun checkAmount(account: String?, amount: Double?, callback: CheckResultCallback) {
        amount ?: return
        account ?: return

        getChecker(account, amount, Consumer { checker ->
            if (amount >= MAX_AMOUNT && checker.isFirstRedPacket) {
                checkerType = CheckerType.Type0
                showDialog(amount, checker, callback)
            } else if (checker.isLastTime(TIME)) {

                if (checker.amount + amount >= ADDITIVE_MAX_AMOUNT && checker.isFirstRedPacketForTime) {
                    checkerType = CheckerType.Type1
                    showDialog(amount, checker, callback)
                } else {
                    checkerType = CheckerType.Type3
                    callback.pass()
                }

            } else {
                checkerType = CheckerType.Type2
                if (amount >= ADDITIVE_MAX_AMOUNT) {
                    showDialog(amount, checker, callback)
                } else {
                    checker.isFirstRedPacketForTime = true
                    callback.pass()
                }
            }

            currentChecker = checker
        })
    }

    fun onPass(amount: Double?) {
        amount ?: return
        val checker = currentChecker ?: return

        when (checkerType) {
            CheckerType.None -> {
                //do nothing
            }
            CheckerType.Type0 -> {
                checker.lastTime = System.currentTimeMillis()
                checker.amount = amount
            }
            CheckerType.Type1 -> {
                checker.isFirstRedPacketForTime = false
                checker.amount += amount
            }
            CheckerType.Type2 -> {
                checker.amount = amount
                checker.lastTime = System.currentTimeMillis()
            }
            else -> {
                if(checker.lastTime == 0L){
                    checker.lastTime = System.currentTimeMillis()
                }
                checker.amount += amount
            }
        }
        checker.isFirstRedPacket = false
        saveChecker(checker)
    }

    fun onCancel() {
        currentChecker = null
        checkerType = CheckerType.None
    }

    private fun showDialog(amount: Double, checker: RedPacketCheckerEntity, callback: CheckResultCallback) {
        val contentLayout = LayoutInflater.from(activity).inflate(R.layout.amount_limit_check, null, false)

        val drawable = activity.resources.getDrawable(R.drawable.message_warning)
        drawable.setBounds(0, 0, drawable.minimumWidth, drawable.minimumHeight)
        val sp = SpannableString(activity.getString(R.string.amount_not_back_guide))
        val imageSpan = CustomImageSpan(drawable, ImageSpan.ALIGN_BOTTOM)
        sp.setSpan(imageSpan, 0, 1, SpannableString.SPAN_EXCLUSIVE_EXCLUSIVE)


        contentLayout.apply {
            titleView.text = sp
            accountView.text = UserInfoHelper.getUserDisplayName(checker.account)
            amountView.text = activity.getString(R.string.yuan_flag, amount.toString())
        }

        BulletAlertCustomViewDialog.Builder(activity).apply {
            setCustomView(contentLayout)
            setTitle(R.string.transaction_reminding)
            hideCloseButton(true)
            setPositiveButton(R.string.confirm_send) { dialog, _ ->
                dialog.dismiss()
                callback.pass()
            }
            setNegativeButton(R.string.cancel) { dialog, _ ->
                dialog.dismiss()
                callback.cancel()
            }
        }.create().show()
    }


    private fun getChecker(account: String?, amount: Double?, consumer: Consumer<RedPacketCheckerEntity>) {
        amount ?: return
        account ?: return
        Observable.fromCallable {
            val rpDao = FlashAppDBHelper.getInstance(NimUIKit.getAccount()).redPacketDao()
            var rpChecker = rpDao.queryRPCheckerByAccount(account).firstOrNull()
            if (rpChecker == null) {
                rpChecker = RedPacketCheckerEntity(account)
                rpChecker.setId(rpDao.insertRPChecker(rpChecker))
            }

            val checker: RedPacketCheckerEntity = rpChecker
            checker
        }.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnEach { loading(false) }
                .subscribe(consumer, Consumer {
                    it.printStackTrace()
                })?.let { loadingDisposableCollector?.collect(it) }
    }

    private fun saveChecker(checker: RedPacketCheckerEntity?) {
        checker ?: return
        Observable.fromCallable {
            FlashAppDBHelper.getInstance(NimUIKit.getAccount()).redPacketDao().updateRPChecker(checker)
        }.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnEach { loading(false) }
                .subscribe({
                    Log.i(TAG, "save checker success!!")
                }, {
                    it.printStackTrace()
                })?.let { loadingDisposableCollector?.collect(it) }
    }

    private fun loading(show: Boolean) {
        loadingDisposableCollector?.loading(show)
    }

    interface CheckResultCallback {
        fun pass()
        fun cancel()
    }

    enum class CheckerType {
        None,
        Type0,
        Type1,
        Type2,
        Type3
    }

}