package com.netease.nim.uikit.business.redpacket.presenter

import android.support.v4.app.FragmentActivity
import android.util.Log
import android.widget.Toast
import com.bullet.libcommonutil.log.sdk.wrapper.NimLog
import com.bullet.redpacket.c2c.grpc.CreateRedPacketOrderRequest
import com.bullet.redpacket.c2c.grpc.RedPacketType
import com.bullet.redpacket.c2c.v2.grpc.CreateOrderRequest
import com.netease.nim.uikit.R
import com.netease.nim.uikit.business.alipay.AliPay
import com.netease.nim.uikit.business.redpacket.AmountChecker
import com.netease.nim.uikit.business.redpacket.activity.SendRedPacketActivity
import com.netease.nim.uikit.business.redpacket.request.RedPacketReqSender
import com.netease.nim.uikit.common.presenter.HandleDisposablePresenter
import com.netease.nim.uikit.common.ui.dialog.DialogMaker
import com.netease.nimlib.sdk.msg.constant.SessionTypeEnum
import com.smartisan.libstyle.toast.NimToast
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.functions.Consumer
import io.reactivex.schedulers.Schedulers
import smartisan.cloud.im.GrpcClient
import smartisan.cloud.im.rxjava.RxGrpcErrorHandler
import java.util.concurrent.TimeUnit

class SendRedPacketPresenter(val sessionId: String,
                             val sessionTypeEnum: SessionTypeEnum) : HandleDisposablePresenter() {
    companion object {
        private val TAG = SendRedPacketPresenter::class.java.simpleName

        const val RED_PACKET_MSG_RESULT = "RED_PACKET_MSG_RESULT"
    }

    private val redpacketSender = RedPacketReqSender()
    private val amountChecker = AmountChecker()

    override fun onAttach(activity: FragmentActivity?) {
        super.onAttach(activity)
        amountChecker.onAttach(activity)
    }

    override fun onDetach() {
        super.onDetach()
        amountChecker.onDetach()
    }

    fun sendRedPacket(money: String?, msg: String?) {
        amountChecker.checkAmount(sessionId, money, object : AmountChecker.CheckResultCallback {
            override fun pass() {
                sendRedPacketInternal(money, msg)
            }

            override fun cancel() {
                amountChecker.onCancel()
            }
        })
    }

    private fun sendRedPacketInternal(money: String?, msg: String?) {
        if (money.isNullOrEmpty()) {
            Log.i(TAG, "send red packet params is null!!")
            NimToast.makeToast(activity, R.string.money_must_not_null, Toast.LENGTH_SHORT).show()
            return
        }
        try {
            if (money?.toDouble() == 0.0) {
                NimToast.makeToast(activity, R.string.money_must_not_null, Toast.LENGTH_SHORT).show()
                return
            }
        } catch (e: NumberFormatException) {
            NimToast.makeToast(activity, R.string.money_format_wrong, Toast.LENGTH_SHORT).show()
            return
        }


        var rpMessage = msg
        if (msg.isNullOrEmpty() || msg.isNullOrBlank()) {
            rpMessage = activity.getString(R.string.red_packet_msg_default)
        }
        rpMessage = rpMessage?.replace("\n", " ")

        val request = CreateRedPacketOrderRequest.newBuilder()
                .setAmount(money)
                .setMessage(rpMessage)
                .setReceivedAccid(sessionId)
                .setRedPacketType(RedPacketType.TYPE_SINGAL)
                .build()

        loading(true)
        AliPay().startPay(request, activity)
                .delay(100L, TimeUnit.MILLISECONDS)
                .subscribeOn(Schedulers.from(GrpcClient.getInstance().executor))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(Consumer { redPacketId ->
                    amountChecker.onPass(money?.toDoubleOrNull())
                    sendPaySuccessToServer(redPacketId)
                    activity.finish()
                }, object : RxGrpcErrorHandler() {

                    override fun error(code: Int, errorMsg: String) {
                        loading(false)
                        NimLog.i(TAG, "ERROR_RESULT:$errorMsg code:$code")
                        NimToast.makeToast(activity, errorMsg, Toast.LENGTH_LONG).show()
                    }

                    override fun error(throwable: Throwable) {
                        loading(false)
                        NimLog.i(TAG, "ERROR_RESULT:$throwable")
                        NimToast.makeToast(activity, throwable.message
                                ?: "支付失败", Toast.LENGTH_LONG).show()
                    }
                })?.let { collectDisposable(it) }
    }

    fun sendRedPacketV2(amount: String?, count: Int?, msg: String?, sessionName: String?, type: com.bullet.redpacket.c2c.v2.grpc.RedPacketTypeV2) {

        if (amount.isNullOrEmpty()) {
            Log.i(TAG, "send red packet params is null!!")
            NimToast.makeToast(activity, R.string.money_must_not_null, Toast.LENGTH_SHORT).show()
            return
        }
        try {
            if (amount?.toDouble() == 0.0) {
                NimToast.makeToast(activity, R.string.money_must_not_null, Toast.LENGTH_SHORT).show()
                return
            }
        } catch (e: NumberFormatException) {
            NimToast.makeToast(activity, R.string.money_format_wrong, Toast.LENGTH_SHORT).show()
            return
        }

        if (count == null || count < 1) {
            NimToast.makeToast(activity, R.string.red_packet_count_wrong, Toast.LENGTH_SHORT).show()
            return
        }


        var rpMessage = msg
        if (msg.isNullOrEmpty() || msg.isNullOrBlank()) {
            rpMessage = activity.getString(R.string.red_packet_msg_default)
        }
        rpMessage = rpMessage?.replace("\n", " ")


        val request = CreateOrderRequest.newBuilder()
                .setAmount(amount)
                .setCount(count)
                .setMessage(rpMessage)
                .setSessionId(sessionId)
                .setSessionType(sessionTypeEnum.value)
                .setRedPacketType(type)
                .setSessionName(sessionName)
                .build()

        loading(true)
        AliPay().startPayV2(request, activity)
                .delay(100L, TimeUnit.MILLISECONDS)
                .subscribeOn(Schedulers.from(GrpcClient.getInstance().executor))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(Consumer { redPacketId ->
                    sendPaySuccessToServer(redPacketId)
                    activity.onBackPressed()
//                    sendPaySuccessToServer()
                }, object : RxGrpcErrorHandler() {

                    override fun error(code: Int, errorMsg: String) {
                        loading(false)
                        NimLog.i(TAG, "ERROR_RESULT:$errorMsg code:$code")
                        NimToast.makeToast(activity, errorMsg, Toast.LENGTH_LONG).show()
                    }

                    override fun error(throwable: Throwable) {
                        loading(false)
                        NimLog.i(TAG, "ERROR_RESULT:$throwable")
                        NimToast.makeToast(activity, throwable.message
                                ?: "支付失败", Toast.LENGTH_LONG).show()
                    }
                })?.let { collectDisposable(it) }
    }

    private fun loading(show: Boolean) {
        activity?.let {
            if (show) {
                DialogMaker.showProgressDialog(it, "", "", true) {
                    clearDisposable()
                }
            } else {
                DialogMaker.dismissProgressDialog()
            }
        }
    }

    private fun sendPaySuccessToServer(redPacketId: String) {
        val sendActivity = activity;
        if (sendActivity is SendRedPacketActivity) {
            sendActivity.sendEventData(true)
        }
        redpacketSender.payNotifyRequest(redPacketId)
                .subscribeOn(Schedulers.from(GrpcClient.getInstance().executor))
                .subscribe({
                    // do nothing
                }, {
                    it.printStackTrace()
                })
                ?.let { collectDisposable(it) }
    }

}