package com.netease.nim.uikit.common.util.media

import android.graphics.Matrix
import android.graphics.SurfaceTexture
import android.net.Uri
import android.view.Surface
import android.view.TextureView
import android.view.ViewGroup
import com.netease.nim.uikit.common.util.WakeLockController
import tv.danmaku.ijk.media.exo.IjkExoMediaPlayer

class BMMediaPlayer(private val videoContainer: ViewGroup) {


    private var oldPlayer: IjkExoMediaPlayer? = null
    private var oldsurface: Surface? = null
    private var oldTextureView: TextureView? = null

    private var vWidth: Int = 0
    private var vHeight: Int = 0
    private var source: String = ""

    //屏幕点亮唤醒
    private val mWakeLockController: WakeLockController = WakeLockController.createInstance(videoContainer.context)

    private val videoStatusListenerAdapterList = mutableListOf<VideoStatusListenerAdapter>()

    var isPause = false
        private set

    /**
     * 开始播放
     * @see #resetAndStart(listenerAdapter: SurfaceTextureListenerAdapter)
     */
    fun resetAndStart() {
        resetAndStart(object : SurfaceTextureListenerAdapter() {
            override fun onSurfaceTextureAvailable(surface: SurfaceTexture?, width: Int, height: Int) {
                super.onSurfaceTextureAvailable(surface, width, height)
                startVideo(Surface(surface), 0)
            }
        })
    }

    /**
     * 1.如果还有未释放的资源那么先释放资源
     * 2.如果已经创建过texture view 那么先移除掉旧的texture view
     * 3.创建新的texture view 并将其添加到界面
     * 4.texture view 初始化完成后开始播放视频
     */
    fun resetAndStart(listenerAdapter: SurfaceTextureListenerAdapter) {
        release()
        isPause = false
        oldTextureView = createTextureView(listenerAdapter)
        videoContainer.addView(oldTextureView)
    }

    fun startVideo(surface: Surface, position: Int) {

        oldPlayer = IjkExoMediaPlayer(videoContainer.context).apply {
            setOnPreparedListener { player ->
                player.seekTo(position.toLong())
                player.start()
                mWakeLockController.acquireWl()
                observerStartListener()
            }

            setOnCompletionListener {
                observerEndListener()
            }


            setOnInfoListener { _, status, _ ->

                if (status == 701) {
                    observerLoadingListener()
                } else if (status == 702) {
                    observerLoadedListener()
                }

                false
            }

            setSurface(surface)
            setDataSource(videoContainer.context, Uri.parse(source))
            prepareAsync()
        }
    }

    /**
     * 如果播放器实例不为空，那么跳到指定位置播放；
     * 如果播放器实例为空，那么认为是第一次播放
     */
    fun startBySeek(position: Int) {
        isPause = false
        if (oldPlayer != null) {
            oldPlayer?.seekTo(position.toLong())
            oldPlayer?.start()
            mWakeLockController.acquireWl()
            observerRestartListener()
        } else {
            oldTextureView = createTextureView(object : SurfaceTextureListenerAdapter() {
                override fun onSurfaceTextureAvailable(surface: SurfaceTexture?, width: Int, height: Int) {
                    super.onSurfaceTextureAvailable(surface, width, height)
                    startVideo(Surface(surface), position)
                }
            })
            videoContainer.addView(oldTextureView)
        }
    }

    fun pause() {
        if (oldPlayer?.isPlaying == true) {
            oldPlayer?.pause()
            isPause = true
            mWakeLockController.releaseWl()
            observerPauseListener()
        }
    }

    fun stop() {
        oldPlayer?.let {
            isPause = false
            it.stop()
            mWakeLockController.releaseWl()
            observerEndListener()
        }
    }

    fun release() {
        isPause = false
        oldsurface?.release()
        oldsurface = null
        oldPlayer?.release()
        oldPlayer = null

        oldTextureView?.let {
            mWakeLockController.releaseWl()
            videoContainer.removeView(oldTextureView)
            oldTextureView = null
        }

    }

    private fun createTextureView(listenerAdapter: SurfaceTextureListenerAdapter): TextureView {
        return TextureView(videoContainer.context).apply {
            listenerAdapter.textureView = this
            listenerAdapter.originalWidth = vWidth
            listenerAdapter.originalHeight = vHeight
            surfaceTextureListener = listenerAdapter
        }
    }

    fun isPlaying(): Boolean = oldPlayer?.isPlaying == true

    fun setSize(width: Int, height: Int) {
        vWidth = width
        vHeight = height
    }

    fun setSource(source: String?) {
        source?.let {
            this.source = source
        }
    }

    fun getCurrentPosition() = oldPlayer?.currentPosition ?: 0L

    fun getBufferPercentage() = oldPlayer?.bufferedPercentage ?: 0

    fun getDuration() = oldPlayer?.duration ?: 0L


    open class SurfaceTextureListenerAdapter : TextureView.SurfaceTextureListener {

        var textureView: TextureView? = null
        var originalWidth: Int = 0
        var originalHeight: Int = 0
        override fun onSurfaceTextureSizeChanged(surface: SurfaceTexture?, width: Int, height: Int) {
        }

        override fun onSurfaceTextureUpdated(surface: SurfaceTexture?) {
        }

        override fun onSurfaceTextureDestroyed(surface: SurfaceTexture?): Boolean {
            return false
        }

        override fun onSurfaceTextureAvailable(surface: SurfaceTexture?, width: Int, height: Int) {
            textureView?.let {
                adjustAspectRatio(it, originalWidth, originalHeight)
            }
        }

        private fun adjustAspectRatio(textureView: TextureView, videoWidth: Int, videoHeight: Int) {
            val viewWidth = textureView.measuredWidth
            val viewHeight = textureView.measuredHeight
            val aspectRatio = videoHeight.toDouble() / videoWidth

            val newWidth: Int
            val newHeight: Int
            if (viewHeight > (viewWidth * aspectRatio).toInt()) {
                newWidth = viewWidth
                newHeight = (viewWidth * aspectRatio).toInt()
            } else {
                newWidth = (viewHeight / aspectRatio).toInt()
                newHeight = viewHeight
            }
            val xoff = (viewWidth - newWidth) / 2
            val yoff = (viewHeight - newHeight) / 2

            val txform = Matrix()
            textureView.getTransform(txform)
            txform.setScale(newWidth.toFloat() / viewWidth, newHeight.toFloat() / viewHeight)
            txform.postTranslate(xoff.toFloat(), yoff.toFloat())
            textureView.setTransform(txform)
        }

    }

    fun addVideoListener(listenerAdapter: VideoStatusListenerAdapter) {
        if (!videoStatusListenerAdapterList.contains(listenerAdapter)) {
            videoStatusListenerAdapterList.add(listenerAdapter)
        }
    }

    private fun observerEndListener() {
        videoStatusListenerAdapterList.forEach {
            it.onEnd()
        }
    }

    private fun observerStartListener() {
        videoStatusListenerAdapterList.forEach {
            it.onStart()
        }
    }

    private fun observerPauseListener() {
        videoStatusListenerAdapterList.forEach {
            it.onPause()
        }
    }

    private fun observerRestartListener() {
        videoStatusListenerAdapterList.forEach {
            it.onRestart()
        }
    }

    private fun observerLoadingListener() {
        videoStatusListenerAdapterList.forEach {
            it.onLoading()
        }
    }

    private fun observerLoadedListener() {
        videoStatusListenerAdapterList.forEach {
            it.onLoaded()
        }
    }

    abstract class VideoStatusListenerAdapter {
        open fun onEnd() {}
        open fun onStart() {}
        open fun onPause() {}
        open fun onRestart() {}
        open fun onLoading() {}
        open fun onLoaded() {}
    }
}